# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os.path
from django import template
from django.templatetags import static


register = template.Library()


style_template = template.Template(
    """{% for filename in files %}
       <link href="{{filename}}" rel="stylesheet" type="text/css">
       {% endfor %}"""
)


script_template = template.Template(
    """{% for filename in files %}
       <script src="{{filename}}" type="text/javascript"></script>
       {% endfor %}"""
)


def tags(context, args, type):
    components = (
        [arg for arg in args if arg in context['node_modules']]
        if args else context['node_modules']
    )
    files = []
    for component in components:
        files.append(component)
        context['node_modules'].remove(component)
    return {'files': [
        static.static(f)
        for f in files
        if os.path.splitext(f)[1][1:] == type
    ]}


@register.inclusion_tag(style_template, takes_context=True)
def yarn_styles(context, *args):
    return tags(context, args, 'css')


@register.inclusion_tag(script_template, takes_context=True)
def yarn_scripts(context, *args):
    return tags(context, args, 'js')



from django.core.files.storage import FileSystemStorage
from django.test import TestCase

from ..yarn import yarn_adapter
from ..finders import NodeModulesFinder
from .. import conf
from .base import BaseYarnCase
import os
import os.path
import shutil


class _MakeDirsTestCase(TestCase):
    """
    Helper to create and clean up test directories.
    """

    def setUp(self):
        super(_MakeDirsTestCase, self).setUp()
        self.created = []

    def tearDown(self):
        super(_MakeDirsTestCase, self).tearDown()
        for name in self.created:
            if os.path.exists(name):
                shutil.rmtree(name)

    def makedirs(self, name):
        """
        Wrap os.makedirs() to delete the created directory on teardown.
        """
        os.makedirs(name)
        self.created.append(name)


class SimpleNodeModulesFinderCase(_MakeDirsTestCase):
    """
    Simple YarnFinder tests, without any packages installed.
    """

    def test_list_nonexistent(self):
        """
        When no Yarn folder exists, just gracefully find nothing.
        """
        finder = NodeModulesFinder()
        self.assertEqual(finder.locations, [])
        self.assertEqual(finder.storages, {})

    def test_list_existent(self, leaf_name='node_modules'):
        """
        If 'node_modules' exists, use it to to find files.
        """
        root = os.path.join(conf.NODE_MODULES_ROOT, leaf_name)
        self.makedirs(root)
        finder = NodeModulesFinder()

        self.assertEqual(finder.locations, [('', root)])

        self.assertEqual(set(finder.storages.keys()), set([root]))
        storage = finder.storages[root]
        self.assertIsInstance(storage, FileSystemStorage)
        self.assertEqual(storage.prefix, '')
        self.assertEqual(storage.location, root)

    def test_list_old_path(self):
        """
        If only the old 'node_modules' folder exists, use it instead.
        """
        self.test_list_existent(leaf_name='node_modules')

    def test_list_both(self):
        """
        If both folders exist, only 'yarn_node_modules' should be used.
        """
        self.makedirs(os.path.join(conf.NODE_MODULES_ROOT, 'node_modules'))
        self.test_list_existent(leaf_name='node_modules')


class NodeModulesFinderCase(BaseYarnCase):
    """Test finding installed with yarn files"""

    def setUp(self):
        super(NodeModulesFinderCase, self).setUp()
        yarn_adapter.install(['jquery@1.9'])
        self.finder = NodeModulesFinder()

    def test_find(self):
        """Test staticfinder find"""
        test_path = os.path.join('jquery', 'jquery.min.js')
        path = self.finder.find(test_path)
        self.assertEqual(path, os.path.join(
            conf.NODE_MODULES_ROOT, 'node_modules', test_path,
        ))

    def test_list(self):
        """Test staticfinder list"""
        test_path = os.path.join('jquery', 'jquery.min.js')
        result = self.finder.list([])
        matched = [
            part for part in result if part[0] == test_path
        ]
        self.assertEqual(len(matched), 1)



from ...yarn import yarn_adapter
from ..base import BaseYarnCommand


class Command(BaseYarnCommand):
    args = 'command'
    help = 'Call yarn in node modules root ({0}).'.format(
        yarn_adapter._node_modules_root)

    def add_arguments(self, parser):
        parser.add_argument('command', nargs='*')

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        args = args or tuple(options.pop('command'))
        if self._is_single_command('install', args):
            self._install([])
        else:
            yarn_adapter.call_bower(args)

    def _is_single_command(self, name, args):
        return len(args) == 1 and args[0] == name



# -*- coding: utf-8 -*-
import os.path
import json
import six
from django.conf import settings
from django.utils.datastructures import OrderedSet


def read_files():
    for component in settings.YARN_INSTALLED_APPS:
        component = component.split('#')[0]
        try:
            with open(os.path.join(
                    settings.NODE_MODULES_ROOT,
                    'node_modules',
                    component,
                    'package.json')) as package_json:
                files = json.load(bower_json).get('files')
                for f in files:
                    yield '%s/%s' % (component, f)
        except FileNotFoundError:
            continue


def node_modules(request):
    return {
        'node_modules': OrderedSet([f for f in read_files()]),
    }



from django.core.management import call_command
from django.conf import settings
from django.test import TestCase
from six import StringIO
from mock import MagicMock
from ..yarn import yarn_adapter, YarnAdapter
from .. import conf
from .base import BaseYarnCase, TEST_NODE_MODULES_ROOT
import os
import shutil


class YarnAdapterCase(TestCase):
    """
    YarnAdapter regression tests.
    """

    def test_create_node_modules_root_subdirs(self):
        """
        create_node_modules_root() creates missing intermediate directories.
        """
        if os.path.exists(TEST_NODE_MODULES_ROOT):
            shutil.rmtree(TEST_NODE_MODULES_ROOT)

        subdir = os.path.join(TEST_NODE_MODULES_ROOT, 'sub', 'dir')
        adapter = YarnAdapter(conf.YARN_PATH, subdir)
        adapter.create_node_modules_root()
        self.assertTrue(os.path.exists(subdir))

        shutil.rmtree(TEST_NODE_MODULES_ROOT)


class YarnInstallCase(BaseYarnCase):
    """Test case for yarn_install management command"""

    def setUp(self):
        super(YarnInstallCase, self).setUp()
        self.apps = settings.YARN_INSTALLED_APPS
        self._original_install = yarn_adapter.install
        yarn_adapter.install = MagicMock()

    def tearDown(self):
        super(YarnInstallCase, self).tearDown()
        yarn_adapter.install = self._original_install

    def test_create_node_modules_root(self):
        """Test create node_modules root"""
        self._remove_node_modules_root()
        call_command('yarn_install')

        self.assertTrue(os.path.exists(TEST_NODE_MODULES_ROOT))

    def test_install(self):
        """Test install yarn packages"""
        call_command('yarn_install')
        yarn_adapter.install.assert_called_once_with(self.apps)


class YarnExistsCase(BaseYarnCase):
    """
    Test yarn exists checker.
    This case need yarn to be installed.
    """

    def setUp(self):
        super(YarnExistsCase, self).setUp()
        self._original_exists = yarn_adapter.is_yarn_exists

    def tearDown(self):
        super(YarnExistsCase, self).tearDown()
        yarn_adapter.is_yarn_exists = self._original_exists

    def test_if_exists(self):
        """Test if yarn exists"""
        self.assertTrue(yarn_adapter.is_yarn_exists())

    def test_if_not_exists(self):
        """Test if yarn not exists"""
        adapter = YarnAdapter('/not/exists/path', TEST_NODE_MODULES_ROOT)
        self.assertFalse(adapter.is_yarn_exists())

    def _mock_exists_check(self):
        """Make exists check return false"""
        yarn_adapter.is_yarn_exists = MagicMock()
        yarn_adapter.is_yarn_exists.return_value = False


class YarnCommandCase(BaseYarnCase):
    """Test case for ./manage.py yarn something command"""

    def setUp(self):
        super(YarnCommandCase, self).setUp()
        self.apps = settings.YARN_INSTALLED_APPS
        self._mock_yarn_adapter()

    def _mock_yarn_adapter(self):
        self._original_install = yarn_adapter.install
        yarn_adapter.install = MagicMock()
        self._orig_call = yarn_adapter.call_yarn
        yarn_adapter.call_yarn = MagicMock()
        self._orig_freeze = yarn_adapter.freeze
        yarn_adapter.freeze = MagicMock()

    def tearDown(self):
        super(YarnCommandCase, self).tearDown()
        yarn_adapter.install = self._original_install
        yarn_adapter.call_yarn = self._orig_call
        yarn_adapter.freeze = self._orig_freeze

    def test_install_without_params(self):
        """Test that yarn install without param identical
        with yarn_install

        """
        call_command('yarn', 'install')
        yarn_adapter.install.assert_called_once_with(
            self.apps)

    def test_install_with_params(self):
        """Test yarn install <something>"""
        call_command('yarn', 'install', 'jquery')
        yarn_adapter.call_yarn.assert_called_once_with(
            ('install', 'jquery'))

    def test_call_to_yarn(self):
        """Test simple call to yarn"""
        call_command('yarn', 'update')
        yarn_adapter.call_yarn.assert_called_once_with(
            ('update',))



from pprint import pformat
from django.core.management.base import BaseCommand
from django.conf import settings
from ..yarn import yarn_adapter
from ..exceptions import YarnNotInstalled


class BaseYarnCommand(BaseCommand):
    """Base management command with yarn support"""

    requires_system_checks = False

    # add fake .options_list for Django>=1.10
    if not hasattr(BaseCommand, 'option_list'):
        option_list = ()

    def handle(self, *args, **options):
        self._check_yarn_exists()
        yarn_adapter.create_node_modules_root()

    def _check_yarn_exists(self):
        """Check yarn exists or raise exception"""
        if not yarn_adapter.is_yarn_exists():
            raise YarnNotInstalled()

    def _install(self, args):
        yarn_adapter.install(settings.YARN_INSTALLED_APPS, *args)



try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

from django.contrib.staticfiles.finders import FileSystemFinder
from django.core.files.storage import FileSystemStorage
from . import conf
import os


class NodeModulesFinder(FileSystemFinder):
    """Find static files installed with yarnpkg"""

    def __init__(self, apps=None, *args, **kwargs):
        self.locations = []
        self.storages = OrderedDict()

        root = self._get_node_modules_location()
        if root is not None:
            prefix = ''
            self.locations.append((prefix, root))

            filesystem_storage = FileSystemStorage(location=root)
            filesystem_storage.prefix = prefix
            self.storages[root] = filesystem_storage

    def _get_node_modules_location(self):
        """
        Return the node modules location, or None if one does not exist.
        """
        path = os.path.join(conf.NODE_MODULES_ROOT, 'node_modules')
        if os.path.exists(path):
            return path



from optparse import make_option
from ..base import BaseYarnCommand


class Command(BaseYarnCommand):
    help = 'Install yarn apps'

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        self._install(args)



import os, sys


def is_executable(path):
    """Check file is executable"""
    if sys.platform == 'win32':
        executable = os.access(path, os.X_OK) or path.lower().endswith(".cmd")
        return os.path.isfile(path) and executable
    return os.path.isfile(path) and os.access(path, os.X_OK)



from django.core.management.base import CommandError


class YarnNotInstalled(CommandError):
    """Custom command error"""

    def __init__(self):
        super(YarnNotInstalled, self).__init__(
            "Yarn not installed, read instruction here - http://yarnpkg.com/",
        )



import os



TEST_PROJECT_ROOT = os.path.abspath(
    os.environ.get('TEST_PROJECT_ROOT', '/tmp/'),
)

BASE_DIR = TEST_PROJECT_ROOT

NODE_MODULES_ROOT = os.path.join(TEST_PROJECT_ROOT, 'node_modules')

STATIC_ROOT = os.path.join(TEST_PROJECT_ROOT, 'yarnpkg_static')

STATIC_URL = '/static/'

YARN_INSTALLED_APPS = (
    'jquery#1.9',
    'underscore',
)

SECRET_KEY = 'iamdjangoyarnpkg'

INSTALLED_APPS = (
    'django_yarnpkg',
)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}



from django.conf import settings
from django.test import TestCase
from django.test.utils import override_settings
from ..yarn import yarn_adapter
import os
import shutil


try:
    TEST_NODE_MODULES_ROOT = os.path.join(
        settings.TEST_PROJECT_ROOT, 'node_modules',
    )
except AttributeError:
    TEST_NODE_MODULES_ROOT = '/tmp/node_modules/'


@override_settings(NODE_MODULES_ROOT=TEST_NODE_MODULES_ROOT)
class BaseYarnCase(TestCase):
    """Base bower test case"""

    def setUp(self):
        yarn_adapter.create_node_modules_root()

    def tearDown(self):
        self._remove_node_modules_root()

    def _remove_components_root(self):
        """Remove components root if exists"""
        if os.path.exists(TEST_NODE_MODULES_ROOT):
            shutil.rmtree(TEST_NODE_MODULES_ROOT)

    def assertCountEqual(self, *args, **kwargs):
        """Add python 2 support"""
        if hasattr(self, 'assertItemsEqual'):
            return self.assertItemsEqual(*args, **kwargs)
        else:
            return super(BaseYarnCase, self).assertCountEqual(*args, **kwargs)



from . import conf, shortcuts, exceptions
from distutils.spawn import find_executable
import os
import subprocess
import sys
import json


class YarnAdapter(object):
    """Adapter for working with yarnpkg"""

    def __init__(self, yarn_path, node_modules_root):
        self._yarn_path = yarn_path
        self._node_modules_root = node_modules_root

    def is_yarn_exists(self):
        """Check is bower exists"""
        if shortcuts.is_executable(self._yarn_path)\
                or find_executable(self._yarn_path):
            return True
        else:
            return False

    def create_node_modules_root(self):
        """Create node modules root if need"""
        if not os.path.exists(self._node_modules_root):
            os.makedirs(self._node_modules_root)

    def call_yarn(self, args):
        """Call yarn with a list of args"""
        proc = subprocess.Popen(
            [self._yarn_path] + list(args),
            cwd=self._node_modules_root)
        proc.wait()

    def install(self, packages, *options):
        """Install packages from yarn"""
        self.call_yarn(['add'] + list(options) + list(packages))

    def _accumulate_dependencies(self, data):
        """Accumulate dependencies"""
        for name, version in data['dependencies'].items():
            if version:
                full_name = '{0}@{1}'.format(name, version)
            else:
                full_name = name

            self._packages.append(full_name)
            self._accumulate_dependencies(params)

    def _parse_package_names(self, output):
        """Get package names in yarn"""
        data = json.loads(output)
        self._packages = []
        self._accumulate_dependencies(data)
        return self._packages

yarn_adapter = YarnAdapter(conf.YARN_PATH, conf.NODE_MODULES_ROOT)



from setuptools import setup, find_packages

version = '6.0.1'

setup(
    name='django-yarnpkg',
    version=version,
    description="Integrate django with yarnpkg",
    long_description=open('README.rst').read(),
    classifiers=[
        'Framework :: Django',
        'Programming Language :: JavaScript',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
    keywords='',
    author='Dominik George',
    author_email='nik@naturalnet.de',
    url='https://edugit.org/nik/django-yarnpkg',
    license='BSD',
    packages=find_packages(exclude=['example']),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        'django',
        'six',
    ],
    entry_points="""
      # -*- Entry points: -*-
      """,
)



# -*- coding: utf-8 -*-
from distutils.spawn import find_executable
import os
import sys
from django.conf import settings

__all__ = ['NODE_MODULES_ROOT', 'YARN_PATH']

NODE_MODULES_ROOT = getattr(settings, 'NODE_MODULES_ROOT', os.path.abspath(os.path.dirname(__name__)))

default_yarn_path = find_executable('yarnpkg') or find_executable('yarn')

YARN_PATH = getattr(settings, 'YARN_PATH', default_yarn_path)
